import os


os.environ["SPIDER_NUM"] = "1"
os.environ["REDIS_HOST"] = "r-j6cc5zw8m3pqom4chmpd.redis.rds.aliyuncs.com"
os.environ["REDIS_PASSWORD"] = "SpiderLinux666"
os.environ["REDIS_PORT"] = "6379"
os.environ["REDIS_DB"] = "0"
os.environ["SCHEDULER_QUEUE_LENGTH"] = "100"
os.environ["STORER_QUEUE_LENGTH"] = "10"
os.environ["LOGSTORE"] = "download_meta"
os.environ["LOGHUB_ACCESS_KEY"] = "LTAI5tH2FLYsBkxcbiLdoYZT"
os.environ["LOGHUB_ACCESS_SECRET"] = "4oD1NVvYfaWiqxkmGx6c2xtpPWEq17"
os.environ["BUCKET"] = "databee-video"
os.environ["ENDPOINT"] = "http://oss-cn-hangzhou.aliyuncs.com"
os.environ["OSS_ACCESS_KEY"] = "LTAI5tH2FLYsBkxcbiLdoYZT"
os.environ["OSS_ACCESS_SECRET"] = "4oD1NVvYfaWiqxkmGx6c2xtpPWEq17"
os.environ["CHUNK_SIZE"] = "1048576"
os.environ["MIN_SIZE"] = "1024"


from cobweb import LauncherPro

app = LauncherPro("test", "test", SPIDER_THREAD_NUM=1)

app.SEEDS = [
    {"url": "https://www.baidu.com"},
    {"url": "https://www.google.com"},
    {"url": "https://www.bing.com"},
    {"url": "https://www.baidu.com?v=1"},
    {"url": "https://www.baidu.com?v=2"},
    {"url": "https://www.baidu.com?v=3"},
    {"url": "https://www.baidu.com?v=4"},
    {"url": "https://www.baidu.com?v=5"}
]

if __name__ == '__main__':
    app.start()



import time
import threading

from abc import ABC, abstractmethod
from cobweb.base import BaseItem, Queue, logger


class Pipeline(threading.Thread, ABC):

    def __init__(
            self,
            stop: threading.Event,
            pause: threading.Event,
            upload: Queue, done: Queue,
            upload_size: int,
            wait_seconds: int
    ):
        super().__init__()
        self._stop = stop
        self._pause = pause
        self._upload = upload
        self._done = done

        self.upload_size = upload_size
        self.wait_seconds = wait_seconds

    @abstractmethod
    def build(self, item: BaseItem) -> dict:
        pass

    @abstractmethod
    def upload(self, table: str, data: list) -> bool:
        pass

    def run(self):
        while not self._stop.is_set():
            if not self._upload.length:
                time.sleep(self.wait_seconds)
                continue
            if self._upload.length < self.upload_size:
                time.sleep(self.wait_seconds)
            status = True
            data_info, seeds = {}, []
            try:
                for _ in range(self.upload_size):
                    item = self._upload.pop()
                    if not item:
                        break
                    seeds.append(item.seed)
                    data = self.build(item)
                    data_info.setdefault(item.table, []).append(data)
                for table, datas in data_info.items():
                    try:
                        self.upload(table, datas)
                    except Exception as e:
                        logger.info(e)
                        status = False
            except Exception as e:
                logger.info(e)
                status = False
            if not status:
                for seed in seeds:
                    seed.params.seed_status = "deal model: fail"
            if seeds:
                self._done.push(seeds)

        logger.info("upload pipeline close!")





import re
import hashlib
from typing import Union
from importlib import import_module


def md5(text: Union[str, bytes]) -> str:
    if isinstance(text, str):
        text = text.encode('utf-8')
    return hashlib.md5(text).hexdigest()


def build_path(site, url, file_type):
    return f"{site}/{md5(url)}.{file_type}"


def format_size(content_length: int) -> str:
    units = ["KB", "MB", "GB", "TB"]
    for i in range(4):
        num = content_length / (1024 ** (i + 1))
        if num < 1024:
            return f"{round(num, 2)} {units[i]}"


def dynamic_load_class(model_info):
    if isinstance(model_info, str):
        if "import" in model_info:
            model_path, class_name = re.search(
                r"from (.*?) import (.*?)$", model_info
            ).groups()
            model = import_module(model_path)
            class_object = getattr(model, class_name)
        else:
            model_path, class_name = model_info.rsplit(".", 1)
            model = import_module(model_path)
            class_object = getattr(model, class_name)
        return class_object
    raise TypeError()


# def download_log_info(item:dict) -> str:
#     return "\n".join([" " * 12 + f"{str(k).ljust(14)}:    {str(v)}" for k, v in item.items()])



import logging


class ColorCodes:
    # Text Reset
    RESET = "\033[0m"

    # Regular Colors
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    PURPLE = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"

    # Bright Colors
    BRIGHT_RED = "\033[91m"
    BRIGHT_GREEN = "\033[92m"
    BRIGHT_YELLOW = "\033[93m"
    BRIGHT_BLUE = "\033[94m"
    BRIGHT_PURPLE = "\033[95m"
    BRIGHT_CYAN = "\033[96m"
    BRIGHT_WHITE = "\033[97m"

    # Background Colors
    BG_RED = "\033[41m"
    BG_GREEN = "\033[42m"
    BG_YELLOW = "\033[43m"
    BG_BLUE = "\033[44m"
    BG_PURPLE = "\033[45m"
    BG_CYAN = "\033[46m"
    BG_WHITE = "\033[47m"

    # Bright Background Colors
    BG_BRIGHT_RED = "\033[101m"
    BG_BRIGHT_GREEN = "\033[102m"
    BG_BRIGHT_YELLOW = "\033[103m"
    BG_BRIGHT_BLUE = "\033[104m"
    BG_BRIGHT_PURPLE = "\033[105m"
    BG_BRIGHT_CYAN = "\033[106m"
    BG_BRIGHT_WHITE = "\033[107m"

    # Text Styles
    BOLD = "\033[1m"
    DIM = "\033[2m"
    ITALIC = "\033[3m"
    UNDERLINE = "\033[4m"
    BLINK = "\033[5m"
    REVERSE = "\033[7m"
    HIDDEN = "\033[8m"


class Log:
    logging.getLogger('oss2.api').setLevel(logging.WARNING)
    logging.basicConfig(
        level=logging.INFO,
        format=f'%(asctime)s %(name)s [%(filename)s:%(lineno)d %(funcName)s]'
               f' %(levelname)s -> %(message)s'
    )
    log = logging.getLogger()

    def set_log_name(self, name):
        self.__class__.log = logging.getLogger(name)

    @property
    def debug(self):
        return self.__class__.log.debug

    @property
    def info(self):
        return self.__class__.log.info

    @property
    def warning(self):
        return self.__class__.log.warning

    @property
    def exception(self):
        return self.__class__.log.exception

    @property
    def error(self):
        return self.__class__.log.error

    @property
    def critical(self):
        return self.__class__.log.critical


logger = Log()






import json
import threading
import time
import traceback
from inspect import isgenerator
from typing import Union, Callable, Mapping

from cobweb.constant import DealModel, LogTemplate
from cobweb.base import (
    Queue, 
    Seed, 
    BaseItem, 
    Request, 
    Response, 
    ConsoleItem,
    logger
)


class Crawler(threading.Thread):

    def __init__(
            self,
            stop: threading.Event,
            pause: threading.Event,
            # launcher_queue: Union[Mapping[str, Queue]],
            get_seed: Callable,
            set_seed: Callable,
            add_seed: Callable,
            delete_seed: Callable,
            upload_data: Callable,
            custom_func: Union[Mapping[str, Callable]],
            thread_num: int,
            max_retries: int,
            time_sleep: int,
    ):
        super().__init__()

        self._stop = stop
        self._pause = pause
        self._get_seed = get_seed
        self._set_seed = set_seed
        self._add_seed = add_seed
        self._delete_seed = delete_seed
        self._upload_data = upload_data

        for func_name, _callable in custom_func.items():
            if isinstance(_callable, Callable):
                self.__setattr__(func_name, _callable)

        self.thread_num = thread_num
        self.time_sleep = time_sleep
        self.max_retries = max_retries

    @staticmethod
    def request(seed: Seed) -> Union[Request, BaseItem]:
        yield Request(seed.url, seed, timeout=5)

    @staticmethod
    def download(item: Request) -> Union[Seed, BaseItem, Response, str]:
        response = item.download()
        yield Response(item.seed, response, **item.to_dict)

    @staticmethod
    def parse(item: Response) -> BaseItem:
        upload_item = item.to_dict
        upload_item["text"] = item.response.text
        yield ConsoleItem(item.seed, data=json.dumps(upload_item, ensure_ascii=False))

    # def get_seed(self) -> Seed:
    #     return self._todo.pop()

    def distribute(self, item, seed):
        if isinstance(item, BaseItem):
            self._upload_data(item)
        elif isinstance(item, Seed):
            self._add_seed(item)
        elif isinstance(item, str) and item == DealModel.poll:
            self._set_seed(seed)
        elif isinstance(item, str) and item == DealModel.done:
            self._delete_seed(seed)
        elif isinstance(item, str) and item == DealModel.fail:
            seed.params.seed_status = DealModel.fail
            self._delete_seed(seed)
        else:
            raise TypeError("yield value type error!")

    def spider(self):
        while not self._stop.is_set():

            seed = self._get_seed()

            if not seed:
                time.sleep(1)
                continue

            elif seed.params.retry > self.max_retries:
                seed.params.seed_status = DealModel.fail
                self._delete_seed(seed)
                continue

            seed_detail_log_info = LogTemplate.log_info(seed.to_dict)

            try:
                request_iterators = self.request(seed)

                if not isgenerator(request_iterators):
                    raise TypeError("request function isn't a generator!")

                iterator_status = False

                for request_item in request_iterators:

                    iterator_status = True

                    if isinstance(request_item, Request):
                        iterator_status = False
                        download_iterators = self.download(request_item)
                        if not isgenerator(download_iterators):
                            raise TypeError("download function isn't a generator")

                        for download_item in download_iterators:
                            iterator_status = True
                            if isinstance(download_item, Response):
                                iterator_status = False
                                logger.info(LogTemplate.download_info.format(
                                    detail=seed_detail_log_info,
                                    retry=seed.params.retry,
                                    priority=seed.params.priority,
                                    seed_version=seed.params.seed_version,
                                    identifier=seed.identifier or "",
                                    status=download_item.response,
                                    response=LogTemplate.log_info(download_item.to_dict)
                                ))
                                parse_iterators = self.parse(download_item)
                                if not isgenerator(parse_iterators):
                                    raise TypeError("parse function isn't a generator")
                                for parse_item in parse_iterators:
                                    iterator_status = True
                                    if isinstance(parse_item, Response):
                                        raise TypeError("upload_item can't be a Response instance")
                                    self.distribute(parse_item, seed)
                            else:
                                self.distribute(download_item, seed)
                    else:
                        self.distribute(request_item, seed)

                if not iterator_status:
                    raise ValueError("request/download/parse function yield value error!")
            except Exception as e:
                logger.info(LogTemplate.download_exception.format(
                    detail=seed_detail_log_info,
                    retry=seed.params.retry,
                    priority=seed.params.priority,
                    seed_version=seed.params.seed_version,
                    identifier=seed.identifier or "",
                    exception=''.join(traceback.format_exception(type(e), e, e.__traceback__))
                ))
                seed.params.retry += 1
                # self._todo.push(seed)
                self._set_seed(seed)
                time.sleep(self.time_sleep * seed.params.retry)
            finally:
                time.sleep(0.1)
        logger.info("spider thread close")

    def run(self):
        for index in range(self.thread_num):
            threading.Thread(name=f"spider_{index}", target=self.spider).start()

